@@ -265,11 +265,13 @@ namespace mars {
         NodeId id;
         if(control->graphics) {
           if(loadGraphics) {
-            id = control->graphics->addDrawObject(*nodeS, visual_rep & 1);
-            if(id) {
-              newNode->setGraphicsID(id);
-              if(!reload) {
-                simNodesReload.back().graphicsID1 = id;
+            if(!nodeS->map.hasKey("noVisual") or (bool)nodeS->map["noVisual"] == false) {
+              id = control->graphics->addDrawObject(*nodeS, visual_rep & 1);
+              if(id) {
+                newNode->setGraphicsID(id);
+                if(!reload) {
+                  simNodesReload.back().graphicsID1 = id;
+                }
               }
             }
           }
@@ -1000,6 +1002,9 @@ namespace mars {
         for(nter = nodes->begin(); nter != nodes->end(); nter++) {
           if(nter->second->getGroupID() == (*jter)) {
             id2 = nter->first;
+            if(!nter->second->isMovable()) {
+              applyFunc(nter->second, params);
+            }
             nodes->erase(nter);
             recursiveHelper(id, params, joints, gids, nodes, applyFunc);
             recursiveHelper(id2, params, joints, gids, nodes, applyFunc);
@@ -1729,9 +1734,24 @@ namespace mars {
                            const std::string &value) {
       iMutex.lock();
       NodeMap::iterator iter;
+
       // todo: cfdir1 is a vector
       iter = simNodes.find(id);
-      if(iter == simNodes.end()) return;
+      if(iter == simNodes.end()) {
+        // hack to attache camera to node:
+        iMutex.unlock();
+        if(control->graphics) {
+          if(matchPattern("*/node", key)) {
+            id = getID(value);
+            unsigned long drawId = 0;
+            if(id) {
+              drawId = getDrawID(id);
+            }
+            control->graphics->attacheCamToNode(1, drawId);
+          }
+        }
+        return;
+      }
       NodeData nd = iter->second->getSNode();
       //// fprintf(stderr, "change: %s %s\n", key.c_str(), value.c_str());
       if(matchPattern("*/position", key)) {
